<?php
/**
 * Open Source Social Network
 *
 * @package   Open Source Social Network
 * @author    Open Social Website Core Team <info@openteknik.com>
 * @copyright 2021 OPEN TEKNIK 
 * @license   OPEN SOURCE SOCIAL NETWORK LICENSE 4.0
 * @link      http://www.opensource-socialnetwork.org/licence
 */
define('AzureADLogin', ossn_route()->com . 'AzureADLogin/');
/**
 * Microsoft AD login Init
 * 
 * @return void
 */
function azure_ad_login_init() {
		ossn_register_page('azuread_login', 'azuread_login_handler');
		ossn_register_com_panel('AzureADLogin', 'settings');
		
		ossn_extend_view('css/ossn.default', 'css/azureadlogin/login');
		ossn_extend_view('css/ossn.admin.default', 'css/azureadlogin/admin');
		ossn_extend_view('forms/login2/before/submit', 'azureadlogin/button');
		
		if(ossn_isAdminLoggedin()) {
				ossn_register_action('azureadlogin/settings', AzureADLogin . 'actions/settings.php');
		}
		if(!ossn_isLoggedin()) {
				ossn_register_action('azuread/login', AzureADLogin . 'actions/login.php');
		}
}
function azuread_get_user($id) {
		if(empty($id)) {
				return false;
		}
		$user   = new OssnUser;
		$search = $user->searchUsers(array(
				'page_limit' => 1,
				'limit' => 1,
				'entities_pairs' => array(
						array(
								'name' => 'com_azuread_login_id',
								'value' => $id
						)
				)
		));
		if($search && isset($search[0]->guid)) {
				return $search[0];
		}
		return false;
}
function azuread_login_handler() {
		require_once(AzureADLogin . 'vendors/oauthad/vendor/autoload.php');
		$component = new OssnComponents;
		$settings  = $component->getSettings('AzureADLogin');
		
		$provider                         = new \TheNetworg\OAuth2\Client\Provider\Azure(array(
				'clientId' => $settings->client_id,
				'clientSecret' => $settings->client_secret,
				'redirectUri' => ossn_site_url('azuread_login')
		));
		$provider->defaultEndPointVersion = TheNetworg\OAuth2\Client\Provider\Azure::ENDPOINT_VERSION_2_0;
		
		$token = $provider->getAccessToken('authorization_code', array(
				'scope' => $provider->scope,
				'code' => input('code')
		));
		$user  = $provider->get($provider->getRootMicrosoftGraphUri($token) . '/v1.0/me', $token);
		if($user && !empty($user['id'])) {
				$ossnuser = azuread_get_user($user['id']);
				if(!$ossnuser) {
						$host  = parse_url(ossn_site_url());
						$host  = str_replace('www.', '', $host['host']);
						$email = $user['id'] . '@adlogin.' . $host;
						
						$username = strtolower(preg_replace('/\s+/', '', $user['givenName']));
						if(strlen($username) <= 4) {
								$username = $username . substr(uniqid(), 5);
						}
						$i = 1;
						while(ossn_user_by_username($username)) {
								$username = $username . '' . $i;
								$i++;
						}
						$password_minimum = ossn_call_hook('user', 'password:minimum:length', false, 6);
						$password         = substr(md5(time()), 0, $password_minimum);
						
						$add             = new OssnUser;
						$add->username   = $username;
						$add->first_name = $user['givenName'];
						$add->last_name  = $user['surname'];
						$add->email      = $email;
						$add->password   = $password;
						$add->validated  = true;
						if($guid = $add->addUser()) {
								$u                             = ossn_user_by_guid($guid);
								$u->data->com_azuread_login_id = $user['id'];
								$u->save();
								
								if($add->Login()) {
										redirect("home");
								}
						} else {
								ossn_trigger_message(ossn_print('azureadlogin:create:error'), 'error');
								redirect(REF);
						}
				} else {
						OssnSession::assign('OSSN_USER', $ossnuser);
						redirect("home");
				}
		}
}
ossn_register_callback('ossn', 'init', 'azure_ad_login_init');